// open help viewer to appropriate topic when user presses F1 or Help key
// called from helpContextOverlay.xul and helpMenuOverlay.xul

const CALENDAR_CONTENT_PACK = "chrome://calendarhelp/locale/help.rdf";
var helpFileURI = CALENDAR_CONTENT_PACK;

const SUNBIRD_ID = "{718e30fb-e89b-41dd-9da7-e25a45638b28}";

var RDF = Components
  .classes['@mozilla.org/rdf/rdf-service;1']
  .getService(Components.interfaces.nsIRDFService);

var RCU = Components
  .classes['@mozilla.org/rdf/container-utils;1']
  .getService(Components.interfaces.nsIRDFContainerUtils);

function pathSet () {
	try {
		var p = Components
			.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
		p.getBranch("capability.policy.default.Navigator.preferenceinternal.").setCharPref("get", "allAccess");
		var b = p.getBranch("calendarhelp.path.");

		var d = Components
			.classes["@mozilla.org/file/directory_service;1"]
			.createInstance(Components.interfaces.nsIProperties);

		var f = d.get("AChrom", Components.interfaces.nsIFile);
		b.setCharPref("program", f.parent.path);

		f = d.get("UChrm", Components.interfaces.nsIFile);
		b.setCharPref("profile", f.parent.path);

		var u = RDF
			.GetDataSource("rdf:chrome")
			.GetTarget(
				RDF.GetResource("urn:mozilla:package:calendar"),
				RDF.GetResource("http://www.mozilla.org/rdf/chrome#baseURL"),
				true)
			.QueryInterface(Components.interfaces.nsIRDFLiteral)
			.Value;

		if (u.substr(0, 4) == "jar:")
			u = u.match(/^jar:([^!]+)!/)[1];
		if (u.substr(0, 9) == "resource:")
			u = d.get("ARes", Components.interfaces.nsIFile).parent.path;
		else if (u.substr(0, 5) == "file:") {
			u = Components
				.classes["@mozilla.org/network/protocol;1?name=file"]
				.getService(Components.interfaces.nsIFileProtocolHandler)
				.getFileFromURLSpec(u)
				.parent.parent.path;
			}
		b.setCharPref("calendar", u);
		}
	catch (e) {dump("pathSet error: " + e + "\n");}
	}

function pathClear () {
	try {
		var p = Components
			.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
		p.getBranch("capability.policy.default.Navigator.preferenceinternal.").clearUserPref("get");
		var b = p.getBranch("calendarhelp.path.");
		b.clearUserPref("program");
		b.clearUserPref("profile");
		b.clearUserPref("calendar");
		}
	catch (e) {dump("pathClear error: " + e + "\n");}
	}


function openHelp(topic, contentPack) {
	if (!topic) {
	  var elem = document.commandDispatcher.focusedElement;
		if (!elem) elem = document.documentElement;
		if (elem) {
		  var id = elem.id;
		  while (elem) { 
		    if (id && id != "") break;
		    elem = elem.parentNode;
		    id = elem.id;
		    }
			if (!id) id = "";
			if (id != "") id = ":" + id;

			var context = RDF.GetDataSourceBlocking("chrome://calendarhelp/content/context.rdf");
		  var res = context.GetSource(
		    RDF.GetResource("http://home.netscape.com/NC-rdf#name"),
		    RDF.GetLiteral(document.documentElement.id + id),
		    true);
			if (!res) res = context.GetSource(
		    RDF.GetResource("http://home.netscape.com/NC-rdf#name"),
		    RDF.GetLiteral(document.documentElement.id + ":" + document.documentElement.id),
		    true);
		  if (res) {
				topic = context.GetTarget(
						res,
						RDF.GetResource("http://home.netscape.com/NC-rdf#topic"),
						true)
					.QueryInterface(Components.interfaces.nsIRDFLiteral)
					.Value;
		    }
			}

//	temporary alert for identifying UI elements...
//	alert("context: " + document.documentElement.id + "  " + id
//		+ "\ntopic:   " + topic);

		}

  helpFileURI = contentPack || helpFileURI;

  var topWindow = locateHelpWindow(helpFileURI);
  if (topWindow) {
		topWindow.focus();
		topWindow.displayTopic(topic);
		}
	else {
		setProductDTD();
		enableLinks();

    const params = Components
			.classes["@mozilla.org/embedcomp/dialogparam;1"]
			.createInstance(Components.interfaces.nsIDialogParamBlock);
    params.SetNumberStrings(2);
    params.SetString(0, helpFileURI);
    params.SetString(1, topic);

    const ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
			.getService(Components.interfaces.nsIWindowWatcher);

		topWindow = ww.openWindow(null, "chrome://help/content/help.xul", "_blank",
			"chrome,all,alwaysRaised,dialog=no", params);

		pathSet();
	  }

	}






// initialize DTD for product names
function setProductDTD() {

	registerProductDTD();

	var ds = Components
		.classes["@mozilla.org/file/directory_service;1"]
		.getService(Components.interfaces.nsIProperties);

	// get path to DTD
	var pdtd = ds.get("AChrom", Components.interfaces.nsIFile);
	pdtd.append("calendarhelpdtd");
	pdtd.append("content");
	pdtd.append("calendarhelpdtd");

	// get DTD file
	var cdtd = pdtd.clone();
	cdtd.append("parent.dtd");

	// get current locale
	var here = Components
		.classes["@mozilla.org/intl/nslocaleservice;1"]
		.getService(Components.interfaces.nsILocaleService)
		.getApplicationLocale()
		.getCategory("NSILOCALE_CTYPE");

	if (cdtd.exists()) {
		// check DTD is for current locale
		var his = Components
			.classes["@mozilla.org/network/file-input-stream;1"]
			.createInstance(Components.interfaces.nsIFileInputStream);
		his.init(cdtd, 1, 292, null);

		var sio = Components
			.classes["@mozilla.org/scriptableinputstream;1"]
			.createInstance(Components.interfaces.nsIScriptableInputStream);
		sio.init(his);

		var ok = (sio.read(cdtd.fileSize - 1)
			.indexOf('thisLocale "' + here + '"') > 0);
		sio.close();
		his.close();

		if (ok) return;
		try {cdtd.remove(false);} catch (e) {}
		}

	// determine whether extension or Sunbird
	var id, isExtension;
	try {
		id = Components
			.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch)
			.getCharPref("app.id");
		isExtension = (id != SUNBIRD_ID);
		}
	catch (e) {
		// no id in Mozilla
		id = "";
		isExtension = true;
		}

	// read from current locale's help.dtd
	var helpdtdurl = Components
		.classes["@mozilla.org/network/standard-url;1"]
		.createInstance(Components.interfaces.nsIStandardURL);
	helpdtdurl.init(1, 0, "chrome://calendarhelp/locale/help.dtd", null, null);

	var ios = Components
		.classes["@mozilla.org/network/io-service;1"]
		.getService(Components.interfaces.nsIIOService);

	var ch = ios.newChannelFromURI(helpdtdurl);

	var sin = Components
		.classes["@mozilla.org/scriptableinputstream;1"]
		.createInstance(Components.interfaces.nsIScriptableInputStream);
	sin.init(ch.open());

	// read the definitions
	var ent = sin.read(sin.available()).split("\n");
	var dtdsym = new Array();
	var dtdtxt = new Array();
	const re = /\w+\s+(\w+)\s+"([\S ]+)"/;
	for (var i = 0; i < ent.length; ++i) {
		var sym = re.exec(ent[i]);
		if (sym) {
			dtdsym.push(sym[1]);
			dtdtxt.push(sym[2]);
			}
		}
	sin.close();

	// write definitions to parent.dtd
	const heading = "<!-- CalendarHelp -->\n";
	cdtd.create(0, 420);
	var dtdfile = Components
		.classes["@mozilla.org/network/file-output-stream;1"]
		.createInstance(Components.interfaces.nsIFileOutputStream);
	dtdfile.init(cdtd, 0x1C, 0x34, 0);
	dtdfile.write(heading, heading.length);

	ent = '<!ENTITY thisLocale "' + here + '">\n';
	dtdfile.write(ent, ent.length);

	ent = '<!ENTITY thisAppID "' + id + '">\n';
	dtdfile.write(ent, ent.length);

	for (var i = 0; i < dtdsym.length; ++i) {
		var sym = dtdsym[i];
		ent = null;
		if (isExtension) {
			if (sym == "_anycal") ent = '<!ENTITY thiscal "' + dtdtxt[i] + '">\n';
			if (sym == "mozcal") ent = '<!ENTITY thismozcal "' + dtdtxt[i] + '">\n';
			if (sym == "_calpref") ent = '<!ENTITY thispref "' + dtdtxt[i] + '">\n';
			}
		else {
			if (sym == "sunbird") ent = '<!ENTITY thiscal "' + dtdtxt[i] + '">\n';
			if (sym == "_mozsunbird") ent = '<!ENTITY thismozcal "' + dtdtxt[i] + '">\n';
			if (sym == "_sunpref") ent = '<!ENTITY thispref "' + dtdtxt[i] + '">\n';
			}
		if (ent) dtdfile.write(ent, ent.length);
		}
	dtdfile.close();
	}


// tweak chrome registry for product DTD
function registerProductDTD() {
	var chrome = RDF.GetDataSource("rdf:chrome");
	var umpr = RDF.GetResource("urn:mozilla:package:root");
	var umpchd = RDF.GetResource("urn:mozilla:package:calendarhelpdtd");
	if (RCU.indexOf(chrome, umpr, umpchd) >= 0) return;
		
	var seq = RCU.MakeSeq(chrome, umpr);
	seq.AppendElement(umpchd);
	chrome.Assert(
		umpchd,
		RDF.GetResource("http://www.mozilla.org/rdf/chrome#baseURL"),
		RDF.GetLiteral("resource:/chrome/calendarhelpdtd/content/calendarhelpdtd/"),
		true);
	}



function setHelpFileURI(rdfURI)
{
  helpFileURI = rdfURI;
}

function locateHelpWindow(contentPack) {
    const windowManagerInterface = Components
        .classes['@mozilla.org/appshell/window-mediator;1'].getService()
        .QueryInterface(Components.interfaces.nsIWindowMediator);
    const iterator = windowManagerInterface.getEnumerator("mozilla:help");
    var topWindow = null;
    var aWindow;

    while (iterator.hasMoreElements()) {
        aWindow = iterator.getNext();
        if (aWindow.getHelpFileURI() == contentPack) {
            topWindow = aWindow;
        }
    }
    return topWindow;
}

function enableLinks() {
	var pref = Components
		.classes["@mozilla.org/preferences-service;1"]
		.getService(Components.interfaces.nsIPrefService)
		.getBranch("network.protocol-handler.expose.");
	var c = null;
	try {c = pref.getBoolPref("chrome");} catch (e) {}
	if (!c) pref.setBoolPref("chrome", true);
	}
